@@ -14,7 +14,7 @@ from datetime import datetime
 import gi
 
 from gtk_switcher.debugger import DebuggerWindow
-from gtk_switcher.presetwindow import PresetWindow
+from gtk_switcher.routing import Routing
 from gtk_switcher.videohubconnection import VideoHubConnection
 from pyatem.hexdump import hexdump
 
@@ -163,6 +163,8 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
         self.window = builder.get_object("main_window")
         self.window.set_application(self.application)
         self.headerbar = builder.get_object("headerbar")
+        self.header_context_stack = builder.get_object("header_context_stack")
+        self.macro_status_index = builder.get_object("macro_status_index")
 
         # Load requested view
         self.mainstack = builder.get_object("mainstack")
@@ -202,6 +204,7 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
         self.connection = AtemConnection(self.on_change, self.on_disconnect, self.on_transfer_progress,
                                          self.on_download_done, self.on_connect, self.on_upload_done,
                                          self.on_upload_progress)
+        self.routing = Routing(self.connection)
 
         if args.ip:
             self.connection.ip = args.ip
@@ -233,123 +236,10 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
 
         self.window.add_accel_group(accel)
 
-        self.preset_models = {
-            "colors": Gio.Menu(),
-            "dsk": Gio.Menu(),
-        }
-        self.preset_submenu = {}
-        self.preset_context = None
-        self.presets = {}
-
-        xdg_config_home = os.path.expanduser(os.environ.get('XDG_CONFIG_HOME', '~/.config'))
-
-        for name in self.preset_models:
-            self.preset_submenu[name] = Gio.Menu()
-            self.preset_models[name].append('Save preset', "app.savepreset")
-            self.preset_models[name].append_section('Presets', self.preset_submenu[name])
-
-            presetdir = os.path.join(xdg_config_home, 'openswitcher', 'presets', name)
-            loaded = {}
-            for existing in glob.glob(os.path.join(presetdir, '*.json')):
-                preset_code = os.path.basename(existing).replace('.json', '')
-                with open(existing, 'r') as handle:
-                    data = json.load(handle)
-                    self.presets[preset_code] = data['fields']
-                    loaded[data['name']] = preset_code
-            ordered = list(sorted(loaded.keys()))
-            for preset_name in ordered:
-                preset_code = loaded[preset_name]
-                mi = Gio.MenuItem.new(preset_name, "app.recallpreset")
-                mi.set_detailed_action(f"app.recallpreset(('{name}', '{preset_code}'))")
-                self.preset_submenu[name].append_item(mi)
-
-        self.presets_colors = builder.get_object('presets_colors')
-        self.presets_colors.set_menu_model(self.preset_models['colors'])
-
-        action = Gio.SimpleAction.new("savepreset", None)
-        action.connect("activate", self.on_save_preset)
-        self.application.add_action(action)
-
-        action = Gio.SimpleAction.new("recallpreset", GLib.VariantType.new("(ss)"))
-        action.connect("activate", self.on_recall_preset)
-        self.application.add_action(action)
-
         GLib.timeout_add_seconds(1, self.on_clock)
 
         Gtk.main()
 
-    def on_save_preset(self, *args):
-        context = self.preset_context.split(':')
-        presets_fields = {
-            "colors": ['color-generator'],
-            "dsk": ['dkey-properties-base', 'dkey-properties'],
-        }
-        preset_fields = presets_fields[context[0]]
-        contents = []
-        for field in preset_fields:
-            ms = self.connection.mixer.mixerstate[field]
-            if isinstance(ms, dict):
-                for key in ms:
-                    if len(context) == 2:
-                        if key != int(context[1]):
-                            continue
-                    sf = ms[key]
-                    s = sf.serialize()
-                    if s is not None:
-                        s['_name'] = field
-                        contents.append(s)
-
-        dialog = PresetWindow(self.window, self.application)
-        response = dialog.run()
-
-        if response == Gtk.ResponseType.CANCEL:
-            return
-
-        preset_name = dialog.get_name()
-        dialog.destroy()
-
-        code = str(uuid.uuid4())
-        self.presets[code] = contents
-        mi = Gio.MenuItem.new(preset_name, "app.recallpreset")
-        mi.set_detailed_action(f"app.recallpreset(('{context[0]}', '{code}'))")
-        self.preset_submenu[context[0]].append_item(mi)
-
-        xdg_config_home = os.path.expanduser(os.environ.get('XDG_CONFIG_HOME', '~/.config'))
-        presetdir = os.path.join(xdg_config_home, 'openswitcher', 'presets', context[0])
-        os.makedirs(presetdir, exist_ok=True)
-        presetfile = os.path.join(presetdir, f'{code}.json')
-        with open(presetfile, 'w') as handle:
-            json.dump({
-                'name': preset_name,
-                'fields': contents,
-            }, handle)
-
-    def on_recall_preset(self, action, parameters):
-        parameters = tuple(parameters)
-        context = parameters[0]
-        pc = self.preset_context.split(':')
-        if pc[0] != context:
-            self.log_aw.warning("Preset restore context does not match menu")
-        presetcode = parameters[1]
-        preset = self.presets[presetcode]
-        cmds = []
-
-        instance_override = []
-        for val in pc[1:]:
-            instance_override.append(int(val))
-        if len(instance_override) == 0:
-            instance_override = None
-
-        for field in preset:
-            classname = field['_name'].title().replace('-', '') + "Field"
-            if hasattr(fieldmodule, classname):
-                fieldclass = getattr(fieldmodule, classname)
-                cmds.extend(fieldclass.restore(field, instance_override=instance_override))
-            else:
-                self.log_aw.error(f"Unknown field in preset: {field['_name']}")
-                return
-        self.connection.mixer.send_commands(cmds)
-
     def on_preview_keyboard_change(self, widget, window, key, modifier):
         if self.disable_shortcuts:
             return
@@ -571,10 +461,6 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
                 else:
                     self.connectionstack.set_visible_child_name("connected")
                 self.log_aw.info("Firmware: {}".format(data.version))
-            elif field == 'time':
-                self.on_time_sync(data)
-            elif field == 'time-config':
-                self.on_timecode_config_change(data)
             elif field == 'input-properties':
                 # Ignore input-properties fields in the initial connection
                 if self.inpr_latch == 0:
@@ -584,28 +470,6 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
                 else:
                     self.on_input_layout_change(data)
                     self.on_camera_layout_change(data)
-            elif field == 'program-bus-input':
-                self.on_program_input_change(data)
-            elif field == 'preview-bus-input':
-                self.on_preview_input_change(data)
-            elif field == 'transition-position':
-                self.on_transition_position_change(data)
-            elif field == 'transition-settings':
-                self.on_transition_settings_change(data)
-            elif field == 'transition-preview':
-                self.on_transition_preview_change(data)
-            elif field == 'key-on-air':
-                self.on_key_on_air_change(data)
-            elif field == 'color-generator':
-                self.on_color_change(data)
-            elif field == 'fade-to-black':
-                self.on_ftb_change(data)
-            elif field == 'fade-to-black-state':
-                self.on_ftb_state_change(data)
-            elif field == 'mixer-effect-config':
-                self.on_mixer_effect_config_change(data)
-            elif field == 'topology':
-                self.on_topology_change(data)
             elif field == 'product-name':
                 self.status_model.set_text(data.name)
                 self.log_aw.info("Mixer model: {}".format(data.name))
@@ -614,27 +478,6 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
                 for me in self.me:
                     me.set_mode(data)
                 self.status_mode.set_text(data.get_label())
-            elif field == 'dkey-properties':
-                self.on_dsk_change(data)
-            elif field == 'supersource-box-properties':
-                self.on_supersource_box_change(data)
-            elif field == 'dkey-state':
-                self.on_dsk_state_change(data)
-            elif field == 'mediaplayer-slots':
-                self.on_mediaplayer_slots_change(data)
-            elif field == 'mediaplayer-file-info':
-                self.on_mediaplayer_file_info_change(data)
-            elif field == 'mediaplayer-selected':
-                self.on_mediaplayer_switcher_source_change(data)
-                self.on_mediaplayer_media_source_change(data)
-            elif field == 'transition-mix':
-                self.on_transition_mix_change(data)
-            elif field == 'transition-dip':
-                self.on_transition_dip_change(data)
-            elif field == 'transition-wipe':
-                self.on_transition_wipe_change(data)
-            elif field == 'transition-dve':
-                self.on_transition_dve_change(data)
             elif field == 'fairlight-master-properties':
                 self.on_fairlight_master_properties_change(data)
             elif field == 'fairlight-audio-input':
@@ -653,46 +496,16 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
                 self.on_audio_mixer_master_properties_change(data)
             elif field == 'audio-mixer-monitor-properties':
                 self.on_audio_monitor_properties_change(data)
-            elif field == 'key-properties-base':
-                self.on_key_properties_base_change(data)
-            elif field == 'key-properties-luma':
-                self.on_key_properties_luma_change(data)
-            elif field == 'key-properties-dve':
-                self.on_key_properties_dve_change(data)
-            elif field == 'key-properties-advanced-chroma':
-                self.on_key_properties_advanced_chroma_change(data)
-            elif field == 'key-properties-advanced-chroma-colorpicker':
-                self.on_key_properties_advanced_chroma_colorpicker_change(data)
-            elif field == 'recording-settings':
-                self.on_stream_recording_setting_change(data)
-            elif field == 'recording-disk':
-                self.on_stream_recording_disks_change(data)
-            elif field == 'recording-status':
-                self.on_stream_recording_status_change(data)
-            elif field == 'recording-duration':
-                self.on_stream_recording_duration_change(data)
             elif field == 'aux-output-source':
                 self.on_aux_output_source_change(data)
                 if data.index in self.aux_follow_audio:
                     self.on_aux_monitor_source_change(data)
-            elif field == 'dkey-properties-base':
-                self.on_dkey_properties_base_change(data)
-            elif field == 'macro-properties':
-                self.on_macro_properties_change(data)
             elif field == 'audio-meter-levels':
                 self.on_audio_meter_levels_change(data)
             elif field == 'fairlight-meter-levels':
                 self.on_fairlight_meter_levels_change(data)
             elif field == 'fairlight-master-levels':
                 self.on_fairlight_master_levels_change(data)
-            elif field == 'streaming-service':
-                self.on_streaming_service_change(data)
-            elif field == 'streaming-audio-bitrate':
-                self.on_streaming_audio_bitrate_change(data)
-            elif field == 'streaming-stats':
-                self.on_streaming_stats_change(data)
-            elif field == 'streaming-status':
-                self.on_streaming_status_change(data)
 
             else:
                 if field == 'time':
@@ -710,6 +523,7 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
                 raise
             self.log_aw.error(f"Exception while disconnected: {e}")
 
+    @field('time')
     def on_time_sync(self, data):
         seconds = data.total_seconds()
         if self.timecode_mode == 0:
@@ -719,6 +533,7 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
             tod = 3600 * t.tm_hour + 60 * t.tm_min + t.tm_sec
             self.timecode_offset = tod - seconds
 
+    @field('time-config')
     def on_timecode_config_change(self, data):
         self.timecode_mode = data.mode
 
@@ -760,6 +575,11 @@ class AtemWindow(SwitcherPage, MediaPage, AudioPage, CameraPage):
         else:
             self.disable_levels()
 
-#    @field('input-properties')
-#    def on_input_properties_changed(self, data):
-#        print(data)
+    @field('macro-record-status')
+    def on_macro_record_status_changed(self, data):
+        self.set_class(self.headerbar, 'recording', data.is_recording)
+        if data.is_recording:
+            self.macro_status_index.set_label(f"Recording macro {data.index}")
+            self.header_context_stack.set_visible_child_name('macro')
+        else:
+            self.header_context_stack.set_visible_child_name('live')
