@@ -4,16 +4,22 @@ import gettext
 import logging
 import re
 from datetime import datetime, timedelta
+from logging import Logger
 
+from gtk_switcher.decorators import field
 from gtk_switcher.layout import LayoutView
+from gtk_switcher.panel_color import ColorPanel
+from gtk_switcher.panel_mediaplayer import MediaPlayerPanel
+from gtk_switcher.panel_supersource import SupersourcePanel
 from pyatem.command import CutCommand, AutoCommand, FadeToBlackCommand, TransitionSettingsCommand, WipeSettingsCommand, \
     TransitionPositionCommand, TransitionPreviewCommand, ColorGeneratorCommand, MixSettingsCommand, DipSettingsCommand, \
     DveSettingsCommand, AudioMasterPropertiesCommand, FairlightMasterPropertiesCommand, DkeyRateCommand, \
     DkeyAutoCommand, DkeyTieCommand, \
     DkeyOnairCommand, ProgramInputCommand, PreviewInputCommand, KeyOnAirCommand, KeyFillCommand, \
     FadeToBlackConfigCommand, RecorderStatusCommand, AuxSourceCommand, StreamingServiceSetCommand, \
-    RecordingSettingsSetCommand, StreamingStatusSetCommand, MediaplayerSelectCommand, StreamingAudioBitrateCommand
-from pyatem.field import TransitionSettingsField, InputPropertiesField, TopologyField
+    RecordingSettingsSetCommand, StreamingStatusSetCommand, MediaplayerSelectCommand, StreamingAudioBitrateCommand, \
+    MacroRecordCommand, MacroActionCommand
+from pyatem.field import TransitionSettingsField, InputPropertiesField, TopologyField, RecordingSettingsField
 import gtk_switcher.stream_data
 
 import gi
@@ -84,6 +90,7 @@ class SwitcherPage:
         self.stream_recorder_switch = builder.get_object('stream_recorder_switch')
         self.stream_recorder_clock = builder.get_object('stream_recorder_clock')
         self.stream_recorder_status = builder.get_object('stream_recorder_status')
+        self.stream_recorder_trigger_cameras = builder.get_object('stream_recorder_trigger_cameras')
         self.stream_recorder_disk = [None, None]
         self.stream_recorder_active = False
         self.stream_recorder_start_time = None
@@ -105,13 +112,14 @@ class SwitcherPage:
         self.stream_live_active = False
         self.stream_live_start_time = None
 
+        self.macro_name = builder.get_object('macro_name')
+
         self.flap = builder.get_object('flap')
         self.flaptoggle = builder.get_object('flaptoggle')
 
         self.flap.bind_property("reveal-flap", self.flaptoggle, "active", GObject.BindingFlags.BIDIRECTIONAL)
 
         self.switcher_mediaplayers = builder.get_object('switcher_mediaplayers')
-        self.mediaplayer_dropdowns = {}
 
         action_streampreset = Gio.SimpleAction.new("streampreset", GLib.VariantType.new("a{sv}"))
         action_streampreset.connect("activate", self.load_livestream_preset)
@@ -167,13 +175,16 @@ class SwitcherPage:
             btn.connect('pressed', self.on_dve_transition_style_clicked)
             self.dve_style_squeeze.append(btn)
 
-        self.color1 = builder.get_object('color1')
-        self.color2 = builder.get_object('color2')
+        self.palette_top = builder.get_object('palette_top')
 
         self.model_me1_fill = builder.get_object('model_me1_fill')
         self.model_key = builder.get_object('model_key')
         self.model_aux = builder.get_object('model_aux')
         self.model_disks = builder.get_object('model_disks')
+        self.model_supersource_box = Gtk.ListStore()
+        self.model_supersource_box.set_column_types([str, str])
+        self.model_supersource_art = Gtk.ListStore()
+        self.model_supersource_art.set_column_types([str, str])
         self.model_changing = False
         self.slider_held = False
 
@@ -301,16 +312,6 @@ class SwitcherPage:
         cmd = TransitionPreviewCommand(index=index, enabled=enabled)
         self.connection.mixer.send_commands([cmd])
 
-    def on_color1_color_set(self, widget):
-        color = widget.get_rgba()
-        cmd = ColorGeneratorCommand.from_rgb(index=0, red=color.red, green=color.green, blue=color.blue)
-        self.connection.mixer.send_commands([cmd])
-
-    def on_color2_color_set(self, widget):
-        color = widget.get_rgba()
-        cmd = ColorGeneratorCommand.from_rgb(index=1, red=color.red, green=color.green, blue=color.blue)
-        self.connection.mixer.send_commands([cmd])
-
     def on_auto_rate_changed(self, widget, index, style, frames):
         if self.model_changing:
             return
@@ -336,8 +337,8 @@ class SwitcherPage:
     def on_aux_source_changed(self, widget):
         if hasattr(widget, 'ignore_change') and widget.ignore_change or self.model_changing:
             return
-        cmd = AuxSourceCommand(widget.index, source=int(widget.get_active_id()))
-        self.connection.mixer.send_commands([cmd])
+
+        self.routing.change(widget.index, int(widget.get_active_id()))
 
     def on_wipe_symmetry_adj_value_changed(self, widget, *args):
         if self.model_changing:
@@ -446,12 +447,14 @@ class SwitcherPage:
         elif len(part) == 2:
             return (int(part[0]) * transition_rate) + int(part[1])
 
+    @field('fade-to-black')
     def on_ftb_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got FTB change for non-existing M/E {}".format(data.index + 1))
             return
         self.me[data.index].set_ftb_rate(data.rate)
 
+    @field('transition-mix')
     def on_transition_mix_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition mix change for non-existing M/E {}".format(data.index + 1))
@@ -461,6 +464,7 @@ class SwitcherPage:
         self.mix_rate.set_text(label)
         self.me[data.index].set_auto_rate('mix', data.rate)
 
+    @field('transition-dip')
     def on_transition_dip_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition dip change for non-existing M/E {}".format(data.index + 1))
@@ -473,6 +477,7 @@ class SwitcherPage:
         self.dip_source.ignore_change = False
         self.me[data.index].set_auto_rate('dip', data.rate)
 
+    @field('transition-wipe')
     def on_transition_wipe_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition wipe change for non-existing M/E {}".format(data.index + 1))
@@ -503,6 +508,7 @@ class SwitcherPage:
         self.set_class(self.wipe_reverse, 'active', data.reverse)
         self.set_class(self.wipe_flipflop, 'active', data.flipflop)
 
+    @field('transition-dve')
     def on_transition_dve_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition dve change for non-existing M/E {}".format(data.index + 1))
@@ -522,78 +528,48 @@ class SwitcherPage:
     def _remap(self, value, old_min, old_max, new_min, new_max):
         return ((value - old_min) / (old_max - old_min)) * (new_max - new_min) + new_min
 
+    @field('dkey-properties')
     def on_dsk_change(self, data):
         self.me[0].set_dsk(data)
-        if data.index in self.dsks:
-            self.dsks[data.index].on_key_properties_change(data)
+        top = 9000 - data.top
+        bottom = data.bottom + 9000
+        left = data.left + 16000
+        right = 16000 - data.right
 
-            top = 9000 - data.top
-            bottom = data.bottom + 9000
-            left = data.left + 16000
-            right = 16000 - data.right
-
-            region = self.layout[0].get(LayoutView.LAYER_DSK, data.index)
-            region.set_mask(top, bottom, left, right)
+        region = self.layout[0].get(LayoutView.LAYER_DSK, data.index)
+        region.set_mask(top, bottom, left, right)
 
+    @field('dkey-state')
     def on_dsk_state_change(self, data):
         self.me[0].set_dsk_state(data)
         region = self.layout[0].get(LayoutView.LAYER_DSK, data.index)
         region.set_tally(data.on_air)
 
-    def make_preset_expander(self, label, context, widget):
-        overlay = Gtk.Overlay()
-        expander = Gtk.Expander()
-        expander.get_style_context().add_class('bmdgroup')
-        frame_label = Gtk.Label(label)
-        frame_label.get_style_context().add_class("heading")
-        expander.set_label_widget(frame_label)
-        expander.set_expanded(True)
-        expander.add(widget)
-        overlay.add(expander)
-
-        button = Gtk.MenuButton()
-        button.set_valign(Gtk.Align.START)
-        button.set_halign(Gtk.Align.END)
-        self.set_class(button, 'flat', True)
-        self.set_class(button, 'preset', True)
-        self.apply_css(button, self.provider)
-        overlay.add_overlay(button)
-
-        hamburger = Gtk.Image.new_from_icon_name('open-menu-symbolic', Gtk.IconSize.BUTTON)
-        button.set_image(hamburger)
-        button.set_name(context)
-
-        context = context.split(':')
-        button.set_menu_model(self.preset_models[context[0]])
-
-        button.connect("clicked", self.on_presets_clicked)
-        return overlay
-
+    @field('topology')
     def on_topology_change(self, data: TopologyField):
         # Create the M/E units
         for i in range(0, data.me_units - len(self.me)):
             self.add_mixeffect()
 
+        # Color generators
+        colorpanel = ColorPanel(self.connection.mixer)
+        colorpanel.window = self.window
+        colorpanel.application = self.application
+        self.apply_css(colorpanel, self.provider)
+        self.palette_top.add(colorpanel)
+
         # Downstream keyer count, only available on M/E 1
         self.me[0].set_topology(data)
         self.apply_css(self.me[0], self.provider)
 
-        from gtk_switcher.downstreamkey import DownstreamKeyer
+        from gtk_switcher.panel_downstreamkeyer import DownstreamKeyerPanel
         for widget in self.downstream_keyers:
             self.downstream_keyers.remove(widget)
 
         for i in range(0, data.downstream_keyers):
-            label = _("Downstream keyer {}").format(i + 1)
-            dsk = DownstreamKeyer(index=i, connection=self.connection)
-            exp = self.make_preset_expander(label, f"dsk:{dsk.index}", dsk)
-
-            self.dsks[dsk.index] = dsk
-            self.has_models.append(dsk)
-            exp.add(dsk)
-            self.apply_css(dsk, self.provider)
-            dsk.set_fill_model(self.model_me1_fill)
-            dsk.set_key_model(self.model_key)
-            self.downstream_keyers.pack_start(exp, False, True, 0)
+            panel = DownstreamKeyerPanel(self.connection.mixer, i, self.model_me1_fill, self.model_key)
+            self.apply_css(panel, self.provider)
+            self.downstream_keyers.pack_start(panel, False, True, 0)
 
             # Add the DSK to the layout editor of M/E 1
             region = self.layout[0].get(LayoutView.LAYER_DSK, i)
@@ -603,71 +579,18 @@ class SwitcherPage:
 
         # Media players
         for i in range(0, data.mediaplayers):
-            label = Gtk.Label(_("Media Player {}").format(i + 1), xalign=0.0)
-            label.get_style_context().add_class('heading')
-            expander = Gtk.Expander()
-            expander.set_label_widget(label)
-            expander.set_expanded(True)
-            frame = Gtk.Frame()
-            frame.set_margin_top(6)
-            frame.get_style_context().add_class('view')
-            expander.add(frame)
-
-            grid = Gtk.Grid()
-            grid.set_column_spacing(12)
-            grid.set_row_spacing(12)
-            grid.set_margin_top(12)
-            grid.set_margin_bottom(12)
-            grid.set_margin_start(12)
-            grid.set_margin_end(12)
-            frame.add(grid)
-
-            label = Gtk.Label("Media", xalign=1.0)
-            label.get_style_context().add_class('dim-label')
-            grid.attach(label, 0, 0, 1, 1)
-
-            dropdown = Gtk.ComboBox.new_with_model(self.model_media)
-            grid.attach(dropdown, 1, 0, 1, 1)
-            dropdown.set_entry_text_column(1)
-            dropdown.set_id_column(0)
-            self.mediaplayer_dropdowns[i] = dropdown
-
-            if 'mediaplayer-selected' in self.connection.mixer.mixerstate:
-                if i in self.connection.mixer.mixerstate['mediaplayer-selected']:
-                    field = self.connection.mixer.mixerstate['mediaplayer-selected'][i]
-                    if field.source_type == 1:
-                        # Stills source
-                        dropdown.set_active_id(str(field.slot))
-
-            dropdown.mediaplayer = i
-            dropdown.connect('changed', self.on_mediaplayer_change)
-            renderer = Gtk.CellRendererText()
-            dropdown.pack_start(renderer, True)
-            dropdown.add_attribute(renderer, "text", 1)
-
-            self.switcher_mediaplayers.add(expander)
+            panel = MediaPlayerPanel(i, model_media=self.model_media, connection=self.connection.mixer)
+            self.switcher_mediaplayers.add(panel)
         self.media_create_mediaplayers(data.mediaplayers)
         self.switcher_mediaplayers.show_all()
 
-    def on_mediaplayer_switcher_source_change(self, data):
-        if data.index not in self.mediaplayer_dropdowns:
-            return
-        self.model_changing = True
-        if data.source_type == 1:
-            self.mediaplayer_dropdowns[data.index].set_active_id(str(data.slot))
-        self.model_changing = False
-
-    def on_mediaplayer_change(self, widget, *args):
-        if self.model_changing:
-            return
-
-        index = widget.get_active_id()
-        if index == "":
-            return
-
-        index = int(index)
-        cmd = MediaplayerSelectCommand(widget.mediaplayer, still=index)
-        self.connection.mixer.send_commands([cmd])
+        # Supersource
+        for i in range(0, data.supersources):
+            sspanel = SupersourcePanel(self.connection.mixer, i, self.model_supersource_box, self.model_supersource_art)
+            sspanel.window = self.window
+            sspanel.application = self.application
+            self.apply_css(sspanel, self.provider)
+            self.palette_top.add(sspanel)
 
     def on_dsk_tie_clicked(self, widget, index, dsk, enabled):
         cmd = DkeyTieCommand(index=dsk, tie=enabled)
@@ -685,6 +608,7 @@ class SwitcherPage:
         cmd = DkeyAutoCommand(index=dsk)
         self.connection.mixer.send_commands([cmd])
 
+    @field('mixer-effect-config')
     def on_mixer_effect_config_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got _MeC for non-existing M/E {}".format(data.index + 1))
@@ -722,6 +646,7 @@ class SwitcherPage:
 
             self.upstream_keyers.show_all()
 
+    @field('fade-to-black-state')
     def on_ftb_state_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got FTB state change for non-existing M/E {}".format(data.index + 1))
@@ -729,19 +654,7 @@ class SwitcherPage:
 
         self.me[data.index].set_ftb_state(data.done, data.transitioning)
 
-    def on_color_change(self, data):
-        r, g, b = data.get_rgb()
-        color = Gdk.RGBA()
-        color.red = r
-        color.green = g
-        color.blue = b
-        color.alpha = 1.0
-
-        if data.index == 0:
-            self.color1.set_rgba(color)
-        else:
-            self.color2.set_rgba(color)
-
+    @field('key-on-air')
     def on_key_on_air_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got key-on-air for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
@@ -749,18 +662,21 @@ class SwitcherPage:
         self.me[data.index].set_key_on_air(data)
         self.layout[data.index].get(LayoutView.LAYER_USK, data.keyer).set_tally(data.enabled)
 
+    @field('transition-preview')
     def on_transition_preview_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition preview change for non-existing M/E {}".format(data.index + 1))
             return
         self.me[data.index].set_preview_transition(data.enabled)
 
+    @field('transition-settings')
     def on_transition_settings_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition settings change for non-existing M/E {}".format(data.index + 1))
             return
         self.me[data.index].set_transition_settings(data)
 
+    @field('transition-position')
     def on_transition_position_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got transition position change for non-existing M/E {}".format(data.index + 1))
@@ -771,6 +687,7 @@ class SwitcherPage:
         cmd = KeyOnAirCommand(index=index, keyer=keyer, enabled=enabled)
         self.connection.mixer.send_commands([cmd])
 
+    @field('key-properties-base')
     def on_key_properties_base_change(self, data):
         if data.index not in self.usks:
             self.log_sw.warning(
@@ -782,6 +699,7 @@ class SwitcherPage:
             return
         self.usks[data.index][data.keyer].on_key_properties_base_change(data)
 
+    @field('key-properties-luma')
     def on_key_properties_luma_change(self, data):
         if data.index not in self.usks:
             self.log_sw.warning(
@@ -793,6 +711,7 @@ class SwitcherPage:
             return
         self.usks[data.index][data.keyer].on_key_properties_luma_change(data)
 
+    @field('key-properties-dve')
     def on_key_properties_dve_change(self, data):
         if data.index not in self.usks:
             self.log_sw.warning(
@@ -811,6 +730,7 @@ class SwitcherPage:
         region.set_region(data.pos_x / 1000, data.pos_y / 1000, width, height)
         region.set_mask(data.mask_top, data.mask_bottom, data.mask_left, data.mask_right)
 
+    @field('key-properties-advanced-chroma')
     def on_key_properties_advanced_chroma_change(self, data):
         if data.index not in self.usks:
             self.log_sw.warning(
@@ -822,6 +742,7 @@ class SwitcherPage:
             return
         self.usks[data.index][data.keyer].on_advanced_chroma_change(data)
 
+    @field('key-properties-advanced-chroma-colorpicker')
     def on_key_properties_advanced_chroma_colorpicker_change(self, data):
         if data.index not in self.usks:
             self.log_sw.warning(
@@ -841,24 +762,20 @@ class SwitcherPage:
         region.set_tally(data.preview)
         region.set_visible(data.cursor)
 
+    @field('program-bus-input')
     def on_program_input_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got program input change for non-existing M/E {}".format(data.index + 1))
             return
         self.me[data.index].program_input_change(data)
 
+    @field('preview-bus-input')
     def on_preview_input_change(self, data):
         if data.index > len(self.me) - 1:
             self.log_sw.warning("Got preview input change for non-existing M/E {}".format(data.index + 1))
             return
         self.me[data.index].preview_input_change(data)
 
-    def on_dkey_properties_base_change(self, data):
-        if data.index not in self.dsks:
-            self.log_sw.warning("Got dkey-properties-base for non-existant keyer {}".format(data.index))
-            return
-        self.dsks[data.index].on_key_properties_base_change(data)
-
     def on_me_program_changed(self, widget, index, source):
         cmd = ProgramInputCommand(index=index, source=source)
         self.connection.mixer.send_commands([cmd])
@@ -867,7 +784,8 @@ class SwitcherPage:
         cmd = PreviewInputCommand(index=index, source=source)
         self.connection.mixer.send_commands([cmd])
 
-    def on_stream_recording_setting_change(self, data):
+    @field('recording-settings')
+    def on_stream_recording_setting_change(self, data: RecordingSettingsField):
         self.expander_stream_recorder.show()
         self.model_changing = True
         self.stream_recorder_filename.set_text(data.filename)
@@ -875,9 +793,11 @@ class SwitcherPage:
 
         self.stream_recorder_disk1.set_active_id(str(data.disk1))
         self.stream_recorder_disk2.set_active_id(str(data.disk2))
+        self.stream_recorder_trigger_cameras.set_active(data.record_in_cameras)
         self.on_update_recording_buttons()
         self.model_changing = False
 
+    @field('recording-disk')
     def on_stream_recording_disks_change(self, data):
         self.model_changing = True
         if data.is_deleted:
@@ -898,6 +818,7 @@ class SwitcherPage:
         self.on_update_recording_buttons()
         self.model_changing = False
 
+    @field('recording-status')
     def on_stream_recording_status_change(self, data):
         self.on_update_recording_buttons()
 
@@ -925,6 +846,7 @@ class SwitcherPage:
         self.set_class(self.stream_recorder_status, 'program', active)
         self.set_class(self.stream_recorder_clock, 'program', active)
 
+    @field('recording-duration')
     def on_stream_recording_duration_change(self, data):
         # This does not get called nearly often enough
         self.stream_recorder_clock.set_text(f'{data.hours}:{data.minutes}:{data.seconds}')
@@ -975,18 +897,24 @@ class SwitcherPage:
         cmd = RecorderStatusCommand(recording=False)
         self.connection.mixer.send_commands([cmd])
 
+    def on_stream_recorder_trigger_cameras_clicked(self, widget, *args):
+        cmd = RecordingSettingsSetCommand(record_in_camera=widget.get_active())
+        self.connection.mixer.send_commands([cmd])
+
     def on_aux_output_source_change(self, source):
-        if source.index not in self.aux:
+        self.routing.aux_changed(source)
+        aux = self.routing.aux_index_to_port_index(source.index)
+        if aux not in self.aux:
             return
 
-        self.aux[source.index].ignore_change = True
-        self.aux[source.index].set_active_id(str(source.source))
-        self.aux[source.index].ignore_change = False
+        self.aux[aux].ignore_change = True
+        self.aux[aux].set_active_id(str(source.source))
+        self.aux[aux].ignore_change = False
 
         for me in self.me:
             if not hasattr(me, 'category'):
                 continue
-            if me.index == source.index:
+            if me.index == aux:
                 me.source_change(source.source)
 
     def on_input_layout_change(self, changed_input):
@@ -1039,9 +967,16 @@ class SwitcherPage:
             if i.available_aux:
                 self.model_aux.append([str(i.index), i.name])
 
+            if i.available_supersource_box:
+                self.model_supersource_box.append([str(i.index), i.name])
+            if i.available_supersource_art:
+                self.model_supersource_art.append([str(i.index), i.name])
+
             if i.port_type == InputPropertiesField.PORT_AUX_OUTPUT:
-                aux_id = i.index - 8001
+                self.routing.add_output(i)
+                aux_id = i.index
                 if aux_id not in self.aux:
+                    # Create AUX combobox in palette
                     self.aux[aux_id] = Gtk.ComboBox.new_with_model(self.model_aux)
                     self.aux[aux_id].set_entry_text_column(1)
                     self.aux[aux_id].set_id_column(0)
@@ -1052,6 +987,7 @@ class SwitcherPage:
                     self.aux[aux_id].add_attribute(renderer, "text", 1)
                     self.grid_aux.attach(self.aux[aux_id], 1, aux_id, 1, 1)
 
+                    # Create options dropdown after the combobox
                     aux_me = Gtk.CheckButton.new_with_label(_("Show as bus"))
                     aux_me.index = i.index
                     aux_me.connect('toggled', self.on_aux_me_enable_toggled)
@@ -1121,7 +1057,9 @@ class SwitcherPage:
         if widget.get_active():
             inputs = self.connection.mixer.mixerstate['input-properties']
             auxsrcs = self.connection.mixer.mixerstate['aux-output-source']
-            auxsrc = auxsrcs[widget.index - 8001]
+            webcam = widget.index == 8200
+            aux = self.routing.port_index_to_aux_index(widget.index)
+            auxsrc = auxsrcs[aux]
             external = []
             output = []
             passthrough = []
@@ -1129,7 +1067,7 @@ class SwitcherPage:
             black = []
             bars = []
             for i in inputs.values():
-                if i.available_aux:
+                if i.available_aux and not webcam or webcam and i.available_usb:
                     if i.port_type == InputPropertiesField.PORT_EXTERNAL:
                         external.append(i)
                     elif i.port_type == InputPropertiesField.PORT_ME_OUTPUT:
@@ -1163,7 +1101,7 @@ class SwitcherPage:
             aux_me.set_inputs([row1, row2])
             aux_me.source_change(auxsrc.source)
             aux_me.connect('source-changed', self.on_aux_me_source_changed)
-            aux_me.index = widget.index - 8001
+            aux_me.index = widget.index
             aux_me.category = 'aux'
             self.apply_css(aux_me, self.provider)
             self.me.append(aux_me)
@@ -1174,14 +1112,14 @@ class SwitcherPage:
                     continue
                 if me.category != 'aux':
                     continue
-                if me.index == widget.index - 8001:
+                if me.index == widget.index:
                     self.me.remove(me)
                     me.destroy()
 
     def on_aux_me_source_changed(self, widget, aux, source):
-        cmd = AuxSourceCommand(aux, source=source)
-        self.connection.mixer.send_commands([cmd])
+        self.routing.change(aux, source)
 
+    @field('macro-properties')
     def on_macro_properties_change(self, data):
         # Clear the macro flow container
         for widget in self.macro_flow:
@@ -1200,17 +1138,40 @@ class SwitcherPage:
                 self.macro_flow.add(button)
         self.macro_flow.show_all()
 
+    def on_macro_run(self, widget):
+        index = widget.index
+        cmd = MacroActionCommand(MacroActionCommand.ACTION_RUN, index=index)
+        self.connection.mixer.send_commands([cmd])
+
+    def on_macro_delete(self, widget):
+        index = widget.index
+        cmd = MacroActionCommand(MacroActionCommand.ACTION_DELETE, index=index)
+        self.connection.mixer.send_commands([cmd])
+
     def on_macro_context(self, widget, event, *args):
+        if event.button == 1:
+            self.on_macro_run(widget)
+
         if event.button != 3:
             return
 
         self.menu = Gtk.Menu()
+
         run_item = Gtk.MenuItem(_("Run macro"))
+        run_item.index = widget.index
+        run_item.connect('activate', self.on_macro_run)
         self.menu.append(run_item)
+
         edit_item = Gtk.MenuItem(_("Edit macro"))
         edit_item.index = widget.index
         edit_item.connect('activate', self.on_macro_edit)
         self.menu.append(edit_item)
+
+        edit_item = Gtk.MenuItem(_("Delete macro"))
+        edit_item.index = widget.index
+        edit_item.connect('activate', self.on_macro_delete)
+        self.menu.append(edit_item)
+
         self.menu.show_all()
         self.menu.popup(None, None, None, None, 0, Gtk.get_current_event_time())
 
@@ -1218,6 +1179,31 @@ class SwitcherPage:
         self.macro_edit = True
         self.connection.mixer.download(0xffff, widget.index)
 
+    def on_macro_create_clicked(self, widget):
+        name = self.macro_name.get_text()
+        if name.strip() == '':
+            return
+
+        # Find a free macro slot for recording
+        slot_index = 0
+        for slot in self.connection.mixer.mixerstate['macro-properties']:
+            mp = self.connection.mixer.mixerstate['macro-properties'][slot]
+            if not mp.is_used:
+                slot_index = slot
+                break
+        else:
+            self.log_sw.error("Could not find a free macro slot to record in")
+            return
+
+        self.log_sw.info(f'Creating macro "{name}" in slot {slot_index}')
+
+        cmd = MacroRecordCommand(slot_index, name, '')
+        self.connection.mixer.send_commands([cmd])
+
+    def on_macro_status_stop_clicked(self, widget):
+        cmd = MacroActionCommand(MacroActionCommand.ACTION_STOP_RECORD)
+        self.connection.mixer.send_commands([cmd])
+
     def bps_to_human(self, bps):
         if bps < 1000:
             return str(bps) + 'b'
@@ -1228,6 +1214,7 @@ class SwitcherPage:
             val = bps / 1000 / 1000
             return f'{val:g}M'
 
+    @field('streaming-service')
     def on_streaming_service_change(self, data):
         self.expander_livestream.show()
         self.expander_encoder.show()
@@ -1238,11 +1225,13 @@ class SwitcherPage:
         self.stream_live_server.set_text(data.url)
         self.stream_live_key.set_text(data.key)
 
+    @field('streaming-audio-bitrate')
     def on_streaming_audio_bitrate_change(self, data):
         self.expander_encoder.show()
         self.audio_rate_min.set_text(self.bps_to_human(data.min))
         self.audio_rate_max.set_text(self.bps_to_human(data.max))
 
+    @field('streaming-status')
     def on_streaming_status_change(self, data):
         starting = data.status == 2
         active = data.status == 4
@@ -1273,6 +1262,7 @@ class SwitcherPage:
 
         self.stream_live_active = active
 
+    @field('streaming-stats')
     def on_streaming_stats_change(self, data):
         self.live_stats.set_text('{:.2f} Mbps'.format(data.bitrate / 1000 / 1000))
 
@@ -1376,6 +1366,7 @@ class SwitcherPage:
     def on_presets_clicked(self, widget, *args):
         self.preset_context = widget.get_name()
 
+    @field('supersource-box-properties')
     def on_supersource_box_change(self, data):
         x = data.x / 100
         y = data.y / 100
