@@ -213,6 +213,59 @@ class TeranexMiniConverterHdmiToSdi12G(WValueProtoConverter):
             0x02: ('on', 'On'),
             0x00: ('off', 'Off'),
         }),
+        # Maybe also 0xd0, or does one actually do left and one right?
+        Field('analog-embedding', (0x00c0, 1), int, "Audio", "Analog Embedding", mapping={
+            0x08: ('12', '1 & 2'),
+            0x09: ('34', '3 & 4'),
+            0x0a: ('56', '5 & 6'),
+            0x0b: ('78', '7 & 8'),
+            0x0c: ('910', '9 & 10'),
+            0x0d: ('1112', '11 & 12'),
+            0x0e: ('1314', '13 & 14'),
+            0x0f: ('1516', '15 & 16'),
+        }),
+        # Maybe also 0xd1, or does one actually do out 1 and one out 2?
+        Field('aesebu-embedding', (0x00c1, 1), int, "Audio", "AES/EBU Embedding", mapping={
+            0x03: ('14', '1 - 4'),
+            0x04: ('58', '5 - 8'),
+            0x05: ('912', '9 - 12'),
+            0x06: ('1316', '13 - 16'),
+        }),
+        Field('conversion', (0x00b3, 1), int, "Video Processing", "Conversion", mapping={
+            0x00: ('auto', 'Auto'),
+            0x01: ('hd', 'Force to HD'),
+            0x02: ('uhd', 'Force to Ultra HD'),
+        }),
+        Field('sdi-level', (0x00b8, 1), int, "Video Processing", "3G Output", mapping={
+            0x04: ('a', 'Level A'),
+            0x00: ('b', 'Level B'),
+        }),
+        Field('convert-60p-5994i', (0x00b7, 1), int, "Video Processing", "Convert 60p to 59.95i", mapping={
+            0x30: ('yes', 'Enabled'),
+            0x10: ('no', 'Disabled'),
+        }),
+        # In the config tool, XLR input format and sample rate converter are separate settings
+        # I suspect it's really a bit mask, probably a bit like as follows:
+        # Bit 0 (LSB) - Audio/Timecode
+        # Bit 1       - Analog/Digital?
+        # Bit 2       - SRC (enabled for analog too)
+        # There is unfortunately some complicated interaction with this and 0x00dc  when setting these
+        Field('xlr-input', (0x00be, 1), int, "Audio", "XLR Input", mapping={
+            0x06: ('analog', 'Analog'),
+            0x00: ('aesebu', 'AES/EBU (No Sample Rate Converter)'),
+            0x04: ('aesebusrc', 'AES/EBU (Sample Rate Converter)'),
+            0x02: ('hdmi', 'HDMI'),
+            0x03: ('timecodehdmi', 'Timecode (Right) with HDMI'),
+            0x07: ('timecodexlr', 'Timecode (Right) with XLR'),
+        }),
+        Field('audio-input', (0x00dc, 1), int, "Audio", "Audio Input", mapping={
+            0x00: ('xlr', 'XLR'),
+            0x02: ('hdmi', 'HDMI'),
+        }),
+        Field('audio1', (0x00e0, 1), int, "Audio", "Analog In 1", mapping='dB'),
+        Field('audio2', (0x00e1, 1), int, "Audio", "Analog In 2", mapping='dB'),
+        Field('aes12', (0x00e2, 1), int, "Audio", "AES/EBU In 1 & 2", mapping='dB'),
+        Field('aes34', (0x00e3, 1), int, "Audio", "AES/EBU In 3 & 4", mapping='dB'),
         Field('dhcp', (0x0087, 1), int, "Network", "IP Setting", mapping={
             0x01: ('dhcp', 'DHCP'),
             0x00: ('static', 'Static IP'),
@@ -257,12 +310,12 @@ class TeranexMiniConverterOpticalToHdmi12G(WValueProtoConverter):
         # Bit 2       - Unknown/N/A?
         # Bit 3       - Consumer/SMPTE 5.1
         Field('xlr-output', (0x00be, 1), int, "Audio", "XLR Output", mapping={
-            0x02: ('smpteanalog', 'Analog - SMPTE 5.1 (L, R, C, LFE, Ls, Rs)'),
-            0x00: ('smpteaesebu', 'AES/EBU - SMPTE 5.1 (L, R, C, LFE, Ls, Rs)'),
-            0x03: ('smptetimecode', 'Timecode (Right) - SMPTE 5.1 (L, R, C, LFE, Ls, Rs)'),
-            0x0a: ('consumeranalog', 'Analog - Consumer 5.1 (L, R, LFE, C, Ls, Rs)'),
-            0x08: ('consumeraesebu', 'AES/EBU - Analog - Consumer 5.1 (L, R, LFE, C, Ls, Rs)'),
-            0x0b: ('consumertimecode', 'Timecode (Right) - Analog - Consumer 5.1 (L, R, LFE, C, Ls, Rs)'),
+            0x02: ('smpteanalog', 'Analog & SMPTE 5.1 (L, R, C, LFE, Ls, Rs)'),
+            0x00: ('smpteaesebu', 'AES/EBU & SMPTE 5.1 (L, R, C, LFE, Ls, Rs)'),
+            0x03: ('smptetimecode', 'Timecode (Right) & SMPTE 5.1 (L, R, C, LFE, Ls, Rs)'),
+            0x0a: ('consumeranalog', 'Analog & Consumer 5.1 (L, R, LFE, C, Ls, Rs)'),
+            0x08: ('consumeraesebu', 'AES/EBU & Consumer 5.1 (L, R, LFE, C, Ls, Rs)'),
+            0x0b: ('consumertimecode', 'Timecode (Right) & Consumer 5.1 (L, R, LFE, C, Ls, Rs)'),
         }),
         # Maybe also 0xc0, or does one actually do left and one right?
         Field('analog-de-embedding', (0x00d0, 1), int, "Audio", "Analog De-embedding", mapping={
@@ -368,6 +421,55 @@ class TeranexMiniConverterQuadSdiTo12GSdi(WValueProtoConverter):
     ]
 
 
+class MultiView4(WValueProtoConverter):
+    PRODUCT = 0xBDD2
+    NAME = "Blackmagic design MultiView 4"
+    NAME_FIELD = 0x50
+
+    # Add more fields!
+    FIELDS = [
+        # Confirm name length
+        # Length of name may technically be a byte or two longer than 61...
+        Field('name', (0x0050, 61), str, "Device", "Name"),
+        # No identify on this unit
+        # No DHCP or current address on this unit
+        Field('address', (0x0094, 4), ipaddress.IPv4Address, "Network", "Address"),
+        Field('netmask', (0x0098, 4), ipaddress.IPv4Address, "Network", "Netmask"),
+        Field('gateway', (0x009c, 4), ipaddress.IPv4Address, "Network", "Gateway"),
+        Field('video-format', (0x00d2, 1), int, "Video Output", "Video Format", mapping={
+            0x04: ('1080i50', '1080i50'),
+            0x0c: ('1080i5994', '1080i59.94'),
+            0x14: ('2160p25', '2160p25'),
+            0x1c: ('2160p2997', '2160p29.97'),
+        }),
+        # SDI Level not available via USB!
+        Field('sd-aspect', (0x00d6, 1), int, "Video Output", "SD aspect ratio", mapping={
+            0x01: ('169', 'Output as 16:9'),
+            0x00: ('43', 'Output as 4:3'),
+        }),
+        Field('screen-layout', (0x00da, 1), int, "Screen Layout", "Views", mapping={
+            0x01: ('solo', 'Solo'),
+            0x00: ('2x2', '2 x 2'),
+        }),
+        Field('borders', (0x00d7, 1), int, 'Overlay Displays', 'Turn on borders', mapping={
+            0x01: ('yes', 'Enable'),
+            0x00: ('no', 'Disable'),
+        }),
+        Field('labels', (0x00d8, 1), int, 'Overlay Displays', 'Turn on labels', mapping={
+            0x01: ('yes', 'Enable'),
+            0x00: ('no', 'Disable'),
+        }),
+        Field('audio-meters', (0x00d5, 1), int, 'Overlay Displays', 'Turn on audio meters', mapping={
+            0x01: ('yes', 'Enable'),
+            0x00: ('no', 'Disable'),
+        }),
+        Field('sdi-tally', (0x00d9, 1), int, 'Overlay Displays', 'Turn on SDI tally', mapping={
+            0x01: ('yes', 'Enable'),
+            0x00: ('no', 'Disable'),
+        }),
+    ]
+
+
 class AtemProductionStudio4k(AtemLegacyProtocol):
     PRODUCT = 0xBD6E
     NAME = "Blackmagic design ATEM Production Studio 4K"
@@ -378,3 +480,17 @@ class AtemProductionStudio4k(AtemLegacyProtocol):
         Field('netmask', (0x0024, 4), ipaddress.IPv4Address, "Network", "Netmask"),
         Field('gateway', (0x0028, 4), ipaddress.IPv4Address, "Network", "Gateway"),
     ]
+
+
+class Atem2MeProductionStudio4k(WValueProtoConverter):
+    PRODUCT = 0xBD87
+    NAME = "Blackmagic design ATEM 2 M/E Production Studio 4K"
+    NAME_FIELD = 0x48
+
+    FIELDS = [
+        # Length of name may technically be a byte or two longer than 31...
+        Field('name', (0x0048, 31), str, "Device", "Name"),
+        Field('address', (0x0020, 4), ipaddress.IPv4Address, "Network", "Address"),
+        Field('netmask', (0x0024, 4), ipaddress.IPv4Address, "Network", "Netmask"),
+        Field('gateway', (0x0028, 4), ipaddress.IPv4Address, "Network", "Gateway"),
+    ]
