@@ -109,6 +109,7 @@ class AtemProtocol:
         'VuMC': 'multiviewer-vu',
         'VuMo': 'multiviewer-vu-opacity',
         'SaMw': 'multiviewer-safe-area',
+        'StMv': 'multiviewer-safe-area-type',
         'LKOB': 'lock-obtained',
         'FTDa': 'file-transfer-data',
         'LKST': 'lock-state',
@@ -130,6 +131,7 @@ class AtemProtocol:
         'RXCC': 'hyperdeck-clip-count',
         'DCPV': 'displayclock-properties',
         'DSTV': 'displayclock-set-time',
+        'FEna': 'fade-to-black-enabled',
     }
 
     FIELDNAME_UNIQUE = {
@@ -170,7 +172,8 @@ class AtemProtocol:
         'multiviewer-vu': struct.Struct('>BB'),
         'multiviewer-safe-area': struct.Struct('>BB'),
         'camera-control-data-packet': struct.Struct('>BBB'),
-        'supersource-box-properties': struct.Struct('>B'),
+        'supersource-box-properties': struct.Struct('>BB'),
+        'supersource-properties': struct.Struct('>B'),
     }
 
     def __init__(self, ip=None, port=9910, usb=None):
@@ -375,6 +378,10 @@ class AtemProtocol:
             # Start next transfer in the queue
             self._transfer_trigger(self.transfer.store)
             return
+        elif key == 'key-properties-dve':
+            preset = contents.mini_preset_key
+            if preset is not None:
+                self._raise('pip-preset', preset)
 
         if key in self.FIELDNAME_UNIQUE:
             idxes = self.FIELDNAME_UNIQUE[key].unpack_from(raw, 0)
